<template>
  <div v-loading="loading" class="overall-chart">
    <div :id="eleId" class="overall_chart" />
    <no-data v-if="isEmpty" />
  </div>
</template>

<script>
import echarts from '@/assets/js/echarts'
import noData from '@/components/noData'
import { throttle } from 'lodash'
const MapField = { 0: '件量(票)', 1: '计费重量(kg)', 2: '收入(万元)', 3: '客户承诺时效达成率(%)', 4: '产品时效达成率(%)', 5: '客诉率(ppm)', 6: '遗失率(ppm)', 7: '损坏率(ppm)' }
const MapType = { 1: '14天', 2: '14周', 3: '6个月' }
const defaultTpl = `<p class="line0">件量(万票)：<span>0</span></p>
              <p class="line1">计费重量(kg)：<span>0</span></p>
              <p class="line2">收入(万元)：<span>0</span></p>
              <p class="line3">客户承诺时效达成率(%)：<span>0</span></p>
              <p class="line4">产品时效达成率(%)：<span>0</span></p>
              <p class="line5">客诉率(ppm)：<span>0</span></p>
              <p class="line6">遗失率(ppm)：<span>0</span></p>
              <p class="line7">损坏率(ppm)：<span>0</span></p>`
export default {
  name: 'ChartTrend',
  components: { noData },
  props: {
    idx: {
      type: String,
      required: false,
      default: ''
    },
    page: {
      type: String,
      required: false,
      default: ''
    }
  },
  data() {
    return {
      loading: false,
      chart: null,
      dateType: 1,
      isEmpty: true,
      seriesData: []
    }
  },
  computed: {
    eleId() {
      return 'overall_chart' + this.idx
    }
  },
  activated() {
    this.chart && this.chart.resize()
  },
  methods: {
    // @获取图表数据
    async getData(param) {
      if (this.loading) return
      this.loading = true
      this.dateType = param.dateType
      const { success, data } = await this.$api.overview
        .overallTrend(param)
        .catch((e) => {
          this.loading = false
          this.$parent.$parent.loading = false
        })
      if (success && data) {
        this.draw(data)
      }
      this.loading = false
      this.$parent.$parent.loading = false
    },
    // @图表绘制逻辑
    draw(data) {
      // 1.判断容器是否加载完
      if (!document.getElementById(this.eleId)) {
        this.isEmpty = true
        return false
      }

      // 2.生成图表配置数据
      // >X轴数据
      const xAxisData = data.consQtyList.map((c) => c.x || 0)
      this.isEmpty = !xAxisData.length
      // >颜色数据
      const colorConfig = ['#FEA637', '#3A7AFB', '#2FC25B', '#9480FF', '#EC97D8', '#49F1F6', '#70A3F5', '#F02121']
      // >图例数据
      const legendData = [
        this.$t('overview.sets'),
        this.$t('overview.countWeight'),
        this.$t('overview.income'),
        this.$t('overview.customerPromisedRatio'),
        this.$t('overview.productTimeRatio'),
        this.$t('overview.customerReportRatio'),
        this.$t('overview.missingRatio'),
        this.$t('overview.damageRatio')
      ]
      // >表格数据源
      const seriesData = [
        {
          name: this.$t('overview.sets'),
          type: 'line',
          symbol: 'emptyCircle',
          symbolSize: 1,
          itemStyle: {
            emphasis: {
              borderColor: '#FEA637',
              borderWidth: 8
            }
          },
          rawData: data.consQtyList,
          data: data.consQtyList.map((c) => c.y || 0)
        },
        {
          name: this.$t('overview.countWeight'),
          type: 'line',
          yAxisIndex: 1,
          symbol: 'emptyCircle',
          symbolSize: 1,
          itemStyle: {
            emphasis: {
              borderColor: '#3A7AFB',
              borderWidth: 8
            }
          },
          rawData: data.meterageWeightQtyList,
          data: data.meterageWeightQtyList.map((c) => this.toFix(c.y || 0))
        },
        {
          name: this.$t('overview.income'),
          type: 'line',
          yAxisIndex: 2,
          symbol: 'emptyCircle',
          symbolSize: 1,
          itemStyle: {
            emphasis: {
              borderColor: '#2FC25B',
              borderWidth: 8
            }
          },
          rawData: data.incomeList,
          data: data.incomeList.map((c) => this.toWan(c.y || 0))
        },
        {
          name: this.$t('overview.customerPromisedRatio'),
          type: 'line',
          yAxisIndex: 3,
          symbol: 'emptyCircle',
          symbolSize: 1,
          itemStyle: {
            emphasis: {
              borderColor: '#9480FF',
              borderWidth: 8
            }
          },
          rawData: data.customerPromiseOverRateList,
          data: data.customerPromiseOverRateList.map((c) => this.toFix(c.y || 0))
        },
        {
          name: this.$t('overview.productTimeRatio'),
          type: 'line',
          yAxisIndex: 4,
          symbol: 'emptyCircle',
          symbolSize: 1,
          itemStyle: {
            emphasis: {
              borderColor: '#EC97D8',
              borderWidth: 8
            }
          },
          rawData: data.productOverRateList,
          data: data.productOverRateList.map((c) => this.toFix(c.y || 0))
        },
        {
          name: this.$t('overview.customerReportRatio'),
          type: 'line',
          yAxisIndex: 5,
          symbol: 'emptyCircle',
          symbolSize: 1,
          itemStyle: {
            emphasis: {
              borderColor: '#49F1F6',
              borderWidth: 8
            }
          },
          rawData: data.complainOverRateList,
          data: data.complainOverRateList.map((c) => this.toFix(c.y || 0))
        },
        {
          name: this.$t('overview.missingRatio'),
          type: 'line',
          yAxisIndex: 6,
          symbol: 'emptyCircle',
          symbolSize: 1,
          itemStyle: {
            emphasis: {
              borderColor: '#70A3F5',
              borderWidth: 8
            }
          },
          rawData: data.lostOverRateList,
          data: data.lostOverRateList.map((c) => this.toFix(c.y || 0))
        },
        {
          name: this.$t('overview.damageRatio'),
          type: 'line',
          yAxisIndex: 7,
          symbol: 'emptyCircle',
          symbolSize: 1,
          itemStyle: {
            emphasis: {
              borderColor: '#F02121',
              borderWidth: 8
            }
          },
          rawData: data.brokenOverRateList,
          data: data.brokenOverRateList.map((c) => this.toFix(c.y || 0))
        }
      ]
      // >计算所有数据组的最小值及索引
      const { min: globalMin, minIndex: globalMinIdx } = this.findMinData(seriesData)
      console.log('min：', globalMin, ', minIndex：', globalMinIdx)
      // >生成Y轴配置项
      const yAxisConfig = seriesData.map((item, index) => {
        return {
          position: index === 0 ? 'left' : 'right',
          type: 'value',
          name: index === 1 ? '最近' + MapType[this.dateType] : null,
          nameLocation: 'end',
          min: Math.min(...item.data), // 获取当前数据中的最小值
          max: Math.max(...item.data), // 获取当前数据中的最大值
          axisLabel: {
            show: false
          },
          nameTextStyle: {
            align: 'right',
            padding: [0, 0, 16, 0],
            fontSize: 12,
            color: '#999'
          },
          splitLine: {
            show: index === globalMinIdx, // 为了统一Y轴分割线样式，需共用某个Y轴分割线，这里应该是全局最小值所在的那个Y轴
            lineStyle: {
              type: 'dashed'
            }
          }
        }
      })

      // 3.渲染图表
      this.renderChart({
        id: this.eleId,
        option: {
          tooltip: {
            trigger: 'axis',
            confine: true,
            enterable: true,
            axisPointer: {
              type: 'line',
              animation: false,
              crossStyle: {
                color: '#999'
              }
            },
            backgroundColor: 'rgba(255,255,255,0.9)',
            formatter: function(params) {
              if (!params.length) return defaultTpl
              let ret = ''
              params.forEach((itm) => {
                const curRealVal = seriesData[itm.seriesIndex].rawData[itm.dataIndex].y
                ret += `<p class="line${itm.seriesIndex}">${MapField[itm.seriesIndex]}：<span>${curRealVal !== null ? itm.value : '-'}</span></p>`
              })
              return `<div class="tooltip-chart">${ret}</div>`
            }
          },
          grid: {
            left: '2%',
            right: '2%',
            bottom: '3%',
            containLabel: true
          },
          color: colorConfig,
          legend: {
            top: 0,
            left: 0,
            right: 20,
            padding: [16, 0, 0, 20],
            data: legendData
          },
          xAxis: {
            type: 'category',
            boundaryGap: true,
            offset: 10,
            axisLine: {
              // onZero: false,
              lineStyle: {
                color: '#ccc'
              }
            },
            axisLabel: {
              rotate: 45,
              fontSize: 12,
              color: '#666'
            },
            data: xAxisData
          },
          yAxis: yAxisConfig,
          series: seriesData
        }
      })
    },
    // @渲染图表及事件
    renderChart(config) {
      const that = this
      that.chart && that.chart.dispose()
      const el = document.getElementById(config.id)
      if (el) {
        that.chart = echarts.init(el)
        that.chart.setOption(config.option)
        // 监听resize事件
        window.addEventListener('resize', this.resizeFn)
        this.$on('hook:beforeDestroy', () => {
          window.removeEventListener('resize', this.resizeFn)
        })
      }
    },
    // @resize事件函数
    resizeFn() {
      // 注意:在使用keep-alive时，处于其他页面时需要屏蔽，只在当前页面执行 resize，不然会导致图表消失
      // console.log(this.$route.path, this.page)
      throttle(() => this.$route.path === this.page && this.chart.resize(), 100)()
    },
    // @获取数据组中最小值及索引
    findMinData(data) {
      const innerMinArr = data.map(item => {
        return Math.min(...item.data)
      })
      const min = Math.min(...innerMinArr)
      const minIndex = innerMinArr.findIndex(c => c === min)
      return { min, minIndex }
    },
    // @数值转单位万
    toWan(data, decimalPlaces = 1) {
      if (data !== null && data !== 'null' && data !== '') {
        if (data >= 10000) {
          return (data / 10000).toFixed(decimalPlaces)
        } else {
          if (Number(data) !== 0) {
            const wanVal = data / 10000
            const idx = [...(wanVal.toString().split('.')[1])].findIndex(c => c > 0)
            return wanVal.toFixed(idx + 1)
          } else {
            return '0.0'
          }
        }
      } else {
        return '0.0'
      }
    },
    // @处理小数点
    toFix(data, decimalPlaces = 1) {
      if (data !== null && data !== 'null' && data !== '') {
        if (Number(data) !== 0) {
          if (data.includes('.')) {
            if (data.split('.')[0] === '0') {
              const idx = [...(data.split('.')[1])].findIndex(c => c > 0)
              return Number(data).toFixed(idx + 1)
            } else {
              return Number(data).toFixed(decimalPlaces)
            }
          } else {
            return Number(data).toFixed(decimalPlaces)
          }
        } else {
          return '0.0'
        }
      } else {
        return '0.0'
      }
    }
  }
}
</script>

<style lang="scss" scoped>
.overall-chart {
  position: relative;
  height: 500px;
}
.overall_chart {
  width: 100%;
  height: 500px;
}
/deep/.x-nodata {
  position: absolute;
  top: 45px;
  left: 0;
  height: 410px;
  min-height: auto;
  background: none;
}
/deep/.tooltip-chart {
  min-width: 180px;
  padding: 0 10px;
  p {
    margin: 10px 0;
    font-size: 14px;
    color: #666;
    span {
      float: right;
      margin-left: 5px;
    }
    &:before {
      position: relative;
      display: inline-block;
      content: "";
      top: -1px;
      width: 7px;
      height: 7px;
      margin-right: 10px;
      border-radius: 50%;
      background: #f5f5f5;
    }
    &.line0:before {
      background: #fea637;
    }
    &.line1:before {
      background: #3a7afb;
    }
    &.line2:before {
      background: #2fc25b;
    }
    &.line3:before {
      background: #9c5af1;
    }
    &.line4:before {
      background: #f570f0;
    }
    &.line5:before {
      background: #49f1f6;
    }
    &.line6:before {
      background: #70a3f5;
    }
    &.line7:before {
      background: #f02121;
    }
  }
  &.auto {
    min-width: auto;
  }
}
</style>
